#include <util/atomic.h> // Pour le macro ATOMIC_BLOCK

#define IN2 6  // Le sens horaire (Clockwise)
#define IN1 5  // Le sens anti-horaire (Counter Clockwise)
#define intrp 13  
// Déclaration des broches de l'encodeur
const int encoderPinA = 3; // Broche d'interruption pour le signal A
const int encoderPinB = 2; // Broche d'interruption pour le signal B

volatile bool stateA = LOW; // État actuel du signal A
volatile bool stateB = LOW; // État actuel du signal B

volatile long posi = 0; // Position de l'encodeur, déclarée comme volatile pour éviter les problèmes de lecture dans les interruptions
long prevT = 0;
float eprev = 0;
float eintegral = 0;
int count1792=0;
// Variables pour l'angle
int targetTicks = 128;
bool increment = true;
int tam = 1;


void setup() {
  Serial.begin(9600);
  
  // Configurer les broches comme entrées
  pinMode(encoderPinA, INPUT);
  pinMode(encoderPinB, INPUT);
    pinMode(intrp, OUTPUT);

  // Attacher les interruptions
  attachInterrupt(digitalPinToInterrupt(encoderPinA), updatePosition, CHANGE);
  attachInterrupt(digitalPinToInterrupt(encoderPinB), updatePosition, CHANGE);

  // Configurer les broches de contrôle du moteur comme sorties
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  
}

void loop() {
  // Convertir l'angle cible en ticks
  

  // PID constants
  float kp =420;
  float kd =4.6;
  float ki =0;
  // Calcul de la différence de temps
  long currT = micros();
  float deltaT = ((float)(currT - prevT)) / (1.0e6); // Conversion en secondes
  prevT = currT;

  // Lire la position dans un bloc atomique pour éviter les erreurs de lecture
  int pos = 0; 
  ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
    pos = posi;
  }
  
  // Calcul de l'erreur
  int e = pos - targetTicks;

  // Dérivée de l'erreur
  float dedt = (e - eprev) / deltaT;

  // Intégrale de l'erreur
  eintegral = eintegral + e * deltaT;

  // Signal de contrôle
  float u = kp * e + kd * dedt + ki * eintegral;

  // Puissance du moteur
  float pwr1 = fabs(u);
  if (pwr1 > 255) {
    pwr1 = 255;
  }
 float pwr2 =map(pwr1,0,255,0,220);
  // Direction du moteur
  int dir = 1;
  if (u < 0) {
    dir = -1;
  }

  // Contrôle du moteur
  setMotor(dir, pwr2, IN1, IN2);

  // Stocker l'erreur précédente
  eprev = e;

  // Afficher les informations pour le débogage
  
  

  Serial.println(pos);
 // Check if the current position is at the target position
if (pos == targetTicks) { count1792++;}

  // If the current position is 0 (starting point)
  if(count1792==2){


 if (increment) {
    posi = targetTicks;
    targetTicks = posi + 128;
    tam++;
      digitalWrite(intrp, HIGH);
            delay(10);

      digitalWrite(intrp, LOW);

      setMotor(0, 0, IN1, IN2);
      delay(200);
    if (tam == 14) {
      increment = false;
    }
  } else {
    posi = targetTicks;
    targetTicks = posi - 128;
    tam--;

  digitalWrite(intrp, HIGH);
            delay(10);

      digitalWrite(intrp, LOW);

     setMotor(0, 0, IN1, IN2);
      delay(200);
    if (tam== 0) {
      increment = true;
    }
  }

count1792=0;

 
}
}

void setMotor(int dir, int pwmVal, int in1, int in2) {
  if (dir == 1) { // Direction avant
    analogWrite(in1, pwmVal);
    analogWrite(in2, 0);
  }
  else if (dir == -1) { // Direction arrière
    analogWrite(in1, 0);
    analogWrite(in2, pwmVal);
  }
  else { // Arrêter
    analogWrite(in1, 0);
    analogWrite(in2, 0);
  }
}

void updatePosition() {
  // Lire les états actuels des signaux A et B
  bool currentStateA = digitalRead(encoderPinA);
  bool currentStateB = digitalRead(encoderPinB);
  
  // Déterminer la direction de rotation
  if (currentStateA == stateA && currentStateB == stateB) return;

  if (currentStateA != stateA) {
    if (currentStateA == HIGH) {
      if (currentStateB == LOW) {
        posi++; // Mouvement dans une direction (avant)
      } else {
        posi--; // Mouvement dans l'autre direction (arrière)
      }
    } else {
      if (currentStateB == HIGH) {
        posi++; // Mouvement dans une direction (avant)
      } else {
        posi--; // Mouvement dans l'autre direction (arrière)
      }
    }
    stateA = currentStateA;
  }

  if (currentStateB != stateB) {
    if (currentStateB == HIGH) {
      if (currentStateA == LOW) {
        posi--; // Mouvement dans l'autre direction (arrière)
      } else {
        posi++; // Mouvement dans une direction (avant)
      }
    } else {
      if (currentStateA == HIGH) {
        posi--; // Mouvement dans l'autre direction (arrière)
      } else {
        posi++; // Mouvement dans une direction (avant)
      }
    }
    stateB = currentStateB;
  }
}